/**** A P P L I C A T I O N   N O T E   ************************************
*
* Title			: DMX-Input for RGBW-LED Control
* Version		: v4.3
* Last updated	: 01.08.2010
* Target		: PC_DIMMER DMX-RGBW-LED Control [ATtiny2313]
* Clock			: 8MHz
*
* Written by Hendrik Hoelscher, http://www.hoelscher-hi.de
* Edited by Christian Noeding, http://www.pcdimmer.de
***************************************************************************
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version2 of 
 the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 General Public License for more details. 

 If you have no copy of the GNU General Public License, write to the 
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

 For other license models, please contact the author.

;***************************************************************************/
#include "main.h"

// ********************* local definitions *********************

enum {IDLE, BREAK, STARTB, STARTADR};				//DMX states

		 uint8_t 	 gDmxState;
		 uint8_t 	*gDmxPnt;
		 uint16_t	 DmxCount;
		 uint16_t	 CurrentDmxByte;


// *************** DMX Reception Initialisation ****************
void init_DMX(void)
{
	DDRD |= (1<<2);
	UBRRH  = 0;
	UBRRL  = ((F_OSC/4000)-1);						//250kbaud, 8N2
	//UCSRC |= (1<<URSEL)|(3<<UCSZ0)|(1<<USBS);
	UCSRC |= (1<<UCSZ1)|(1<<UCSZ0)|(1<<USBS);
	UCSRB |= (0<<UCSZ2)|(1<<RXEN)|(1<<RXCIE);
	gDmxState= IDLE;
	DmxAlive=255;
}


// *************** DMX Reception ISR ****************
ISR (USART_RX_vect)
{
	uint8_t USARTstate= UCSRA;						//get state
	uint8_t DmxByte= UDR;							//get data
	uint8_t DmxState= gDmxState;					//just get once from SRAM!!!
	 
	if (USARTstate &(1<<FE))						//check for break
	{
		UCSRA &= ~(1<<FE);							//reset flag
		DmxCount= DmxAddress;						//reset frame counter
		CurrentDmxByte=0;
		gDmxState= BREAK;
	}
	else if (DmxState == BREAK)
	{
		if (DmxByte == 0) 
		{
			gDmxState= STARTB;						//normal start code detected
			gDmxPnt= ((uint8_t*)DmxField +1);
		}
		else gDmxState= IDLE;
	}
	else if (DmxState == STARTB)
	{
		if (--DmxCount == 0)						//start address reached?
		{
			gDmxState= STARTADR;
			#ifdef InvertOutput
				DmxField[0]= 255-DmxByte;
			#else
				DmxField[0]= DmxByte;
			#endif
		}
	}
	else if (DmxState == STARTADR)
	{
		uint8_t *DmxPnt;
		DmxPnt= gDmxPnt;
		#ifdef InvertOutput
			*DmxPnt= 255-DmxByte;
		#else
			*DmxPnt= DmxByte;
		#endif
		if (++DmxPnt >= (DmxField +DMX_CHANNELS)) 	//all ch received?
		{
			gDmxState= IDLE;
		}
		else
		{
			gDmxPnt= DmxPnt;
		}
	}

	if ( (DmxState==STARTADR) | (DmxState==IDLE))
	{
		CurrentDmxByte++;

		if (GetNewDMXAddress==TRUE)
		{
			if (CurrentDmxByte==511 && Catched512==FALSE)
			{
				NewAddressAbove255=DmxByte;
				Catched511=TRUE;
			}
			if (CurrentDmxByte==512 && Catched511==TRUE)
			{
				if (NewAddressAbove255>0)
				{
					DmxAddress=DmxByte+256;
				}
				else
				{
					DmxAddress=DmxByte+1;
				}
				GetNewDMXAddress=FALSE;
				Catched512=FALSE;
			}
		}
	}
	DmxAlive=255;							
}
