/**** A P P L I C A T I O N   N O T E   ************************************
*
* Title			: DMX-Input for RGBW-LED Control
* Version		: v4.3
* Last updated	: 03.08.2010
* Target		: PC_DIMMER DMX-RGBW-LED Control [ATtiny2313]
* Clock			: 8MHz
*
* Written by Christian Noeding, http://www.pcdimmer.de
***************************************************************************
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version2 of 
 the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 General Public License for more details. 

 If you have no copy of the GNU General Public License, write to the 
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

 For other license models, please contact the author.

;***************************************************************************/
#include "main.h"


void init_pcdimmer(void)
{
	_delay_ms(250);
	DDRD = (0<<TASTER)|(1<<LEDGREEN)|(1<<LEDYELLOW);	// Taster-Pin als Eingang, sowie LED-Pins als Ausgang definieren

	PORTD |= ( 1 << TASTER );					// Pullups fr Taster einschalten

	ledGreenOff();								// Grne LED ausschalten
	ledYellowOn();								// Gelbe LED einschalten
	_delay_ms(250);
	ledYellowOff();								// Gelbe LED ausschalten
	ledGreenOn();								// Grne LED einschalten
	_delay_ms(250);

	// DMX Adresse zunchst festlegen
	DmxAddress=1;
	
	// DMX-Startwerte
	DmxField[0]=0;
	DmxField[1]=0;
	DmxField[2]=0;
	DmxField[3]=0;
}

void CatchAddressByDmx(void)
{
	DmxAddress=1;

	Catched511=FALSE;
	Catched512=FALSE;
	NewAddressAbove255=FALSE;
	GetNewDMXAddress=TRUE;

	// Warten, bis neue Adresse empfangen
	ledGreenOff();								// Grne LED ausschalten
	ledYellowOff();								// Gelbe LED ausschalten
	while ( !((Catched511==TRUE) && (Catched512==TRUE)) )
	{
		ledGreenToggle();						// Grne LED toggeln
		_delay_ms(50);
	}
					
	eeprom_write_word((uint16_t*) 0x78, DmxAddress);
				
	// Warten, bis User die Taste loslsst
	ledGreenOff();								// Grne LED ausschalten
	ledYellowOff();								// Gelbe LED ausschalten
	while ( !(PIND & (1<<TASTER)) )
	{
		// Taster noch gedrckt
		ledGreenToggle();						// Grne LED toggeln
		ledYellowToggle();						// Gelbe LED toggeln
		_delay_ms(50);
	}
	ledGreenOn();								// Grne LED einschalten
}

int main(void)
{
	cli();
	
	init_pcdimmer();							// PC_DIMMER Hardware initialisieren

	DmxAddress=eeprom_read_word((uint16_t*) 0x78);
	if ( (DmxAddress<1) | (DmxAddress>512) )
	{
		DmxAddress=1;							// Fehlerhafte Startadresse kompensieren
	}
	init_DMX();									// DMX512-Empfang initialisieren

	if ( !(PIND & (1<<TASTER)) )
	{
		// Taste gedrckt -> DMX-Startadresse lesen
		sei();
		CatchAddressByDmx();
		cli();
	};

	init_PWM();									// Pulsweitenmodulation initialisieren
	sei();										// Interrupts aktivieren
	ledGreenOn();								// Grne LED einschalten

	for(;;)
	{
		update_PWM();							// PWM-Register mit Hilfe von Tabelle aktualisieren

		if (DmxAlive==0)
		{
			// Fehler in DMX-Signal
			ledYellowToggle();					// Gelbe LED toggeln
			_delay_ms(50);
		}
		else
		{
			// DMX-Signal OK
			if (DmxAlive==255)
			{
				// Prfen, ob ein Kanal verndert wurde
				for (i=0; i<DMX_CHANNELS; i++){
					if (DmxOldField[i]!=DmxField[i]){
						DmxValueChanged=1;
						DmxOldField[i]=DmxField[i];
					}
				}
				// Sofern neue Daten eingegangen sind, dies anzeigen
				if (DmxValueChanged==1){
					DmxValueChanged=0;
					ledYellowOn();
					_delay_ms(10);
					ledYellowOff();
				}else{
					ledYellowOff();				// Gelbe LED einschalten
				}
			}
			DmxAlive--;

			// Adressnderung whrend der Laufzeit
			if ( !(PIND & (1<<TASTER)) )
			{
				// Taste gedrckt
				count=0;
				while ( !(PIND & (1<<TASTER)) && (count<10) )
				{
					ledGreenToggle();			// Grne LED toggeln
					count++;
					_delay_ms(150);
				}
				if ( !(PIND & (1<<TASTER)) )
				{
					// Taste weiterhin gedrckt -> neue Adresse holen
					CatchAddressByDmx();
				}
				else
				{
					// Taster vorzeitig losgelassen
					ledGreenOn();				// Grne LED einschalten
				}
			}
		}
	};
}
